import json
import os
import pathlib
import subprocess
import time

import pytest
import yaml

from reptor.api.ProjectsAPI import ProjectsAPI
from reptor.api.ProjectDesignsAPI import ProjectDesignsAPI
from reptor.api.NotesAPI import NotesAPI
from reptor.lib.reptor import Reptor


@pytest.mark.integration
@pytest.fixture(scope="session", autouse=True)
def setUp():
    # Rename config file
    filename_timestamp = int(time.time())
    try:
        os.rename(
            pathlib.Path.home() / ".sysreptor/config.yaml",
            pathlib.Path.home() / f".sysreptor/config.{filename_timestamp}.yaml",
        )
    except FileNotFoundError:
        pass

    # Set API Token and URL
    conf()

    # Get existing project id
    p = subprocess.Popen(
        ["reptor", "project", "--json"],
        stdout=subprocess.PIPE,
    )
    projects, _ = p.communicate()
    projects = json.loads(projects.decode())
    assert p.returncode == 0
    project_id = projects[-1]["id"]

    # Also add project_id
    conf(project_id=project_id)

    yield

    # Remove integration test config file
    try:
        os.remove(pathlib.Path.home() / ".sysreptor/config.yaml")
    except FileNotFoundError:
        pass
    # Restore config file
    try:
        os.rename(
            pathlib.Path.home() / f".sysreptor/config.{filename_timestamp}.yaml",
            pathlib.Path.home() / ".sysreptor/config.yaml",
        )
    except FileNotFoundError:
        pass


@pytest.fixture(scope="session")
def projects_api():
    reptor = Reptor()
    return ProjectsAPI(reptor=reptor)


@pytest.fixture(scope="session")
def notes_api():
    reptor = Reptor()
    reptor._config._raw_config["cli"] = {"private_note": False}
    return NotesAPI(reptor=reptor)


@pytest.fixture(scope="session")
def private_notes_api():
    reptor = Reptor()
    reptor._config._raw_config["cli"] = {"private_note": True}
    return NotesAPI(reptor=reptor)


@pytest.fixture(scope="session")
def project_design_api():
    reptor = Reptor()
    reptor._config._raw_config["cli"] = {"private_note": False}
    return ProjectDesignsAPI(reptor=reptor)


@pytest.fixture(scope="module")
def uploads_id(notes_api):
    notes_api.write_note("Create Note")
    uploads_note = get_note("Uploads", None)
    assert uploads_note is not None
    return uploads_note["id"]


@pytest.fixture(scope="module")
def private_uploads_id(private_notes_api):
    private_notes_api.write_note("Create Note")
    uploads_note = get_note("Uploads", None, private=True)
    assert uploads_note is not None
    return uploads_note["id"]


@pytest.fixture(autouse=True, scope="session")
def delete_notes(setUp, notes_api):
    yield

    # Delete all notes via notes_api
    for note in get_notes():
        notes_api.delete_note(note["id"])
    # Assert notes are gone
    notes = get_notes()
    assert len(notes) == 0


@pytest.fixture(autouse=True, scope="session")
def delete_findings(setUp, projects_api):
    yield

    # Delete findings via projects_api
    for finding in projects_api.get_findings():
        projects_api.delete_finding(finding.id)
    # Assert findings are gone
    findings = projects_api.get_findings()
    assert len(findings) == 0


def get_notes(private=False):
    cmd = ["reptor", "note", "--list", "--json"]
    if private:
        cmd.append("--private-note")

    p = subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
    )
    notes, _ = p.communicate()
    notes = json.loads(notes.decode())
    assert p.returncode == 0
    return notes


def get_note(name, parent, notes=None, private=False):
    if notes is None:
        notes = get_notes(private=private)
    if parent:
        pass
    for note in notes:
        if note["title"] == name and note["parent"] == parent:
            return note


def read_until(f, eof=": "):
    content = ""
    while char := f.read(1).decode():
        content += char
        if content.endswith(eof):
            break
    return content


def conf(project_id=""):
    # Test reptor conf and thereby setup config file
    p = subprocess.Popen(
        ["reptor", "conf"],
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        # stderr=subprocess.PIPE,
    )
    read_until(p.stdout)
    p.stdin.write(os.environ.get("SYSREPTOR_SERVER", "").encode("utf-8") + b"\n")  # type: ignore
    p.stdin.flush()  # type: ignore

    read_until(p.stdout)
    p.stdin.write(os.environ.get("SYSREPTOR_API_TOKEN", "").encode("utf-8") + b"\n")  # type: ignore
    p.stdin.flush()  # type: ignore

    read_until(p.stdout)
    p.stdin.write(project_id.encode("utf-8") + b"\n")  # type: ignore
    p.stdin.flush()  # type: ignore

    read_until(p.stdout)
    p.stdin.write(b"y\n")  # type: ignore
    p.stdin.flush()  # type: ignore
    p.wait(timeout=5)

    # Assert config file was created correctly
    with open(pathlib.Path.home() / ".sysreptor/config.yaml") as f:
        config = yaml.safe_load(f)
    assert config["server"] == os.environ.get("SYSREPTOR_SERVER", "")
    assert config["token"] == os.environ.get("SYSREPTOR_API_TOKEN", "")
    if project_id:
        assert config["project_id"] == project_id
    else:
        assert config["project_id"] is None



import typing
from contextlib import contextmanager
from functools import cached_property
from posixpath import join as urljoin
from typing import Optional

from requests import HTTPError

from reptor.api.APIClient import APIClient
from reptor.models.Finding import FindingRaw
from reptor.models.Project import Project, ProjectOverview
from reptor.models.Section import Section, SectionRaw


class ProjectsAPI(APIClient):
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)
        self._init_attrs()

    def _init_attrs(self) -> None:
        self.project_design = None

        if not (server := self.reptor.get_config().get_server()):
            raise ValueError("No SysReptor server configured. Try 'reptor conf'.")

        self.base_endpoint = f"{server}/api/v1/pentestprojects"
        self.debug(self.base_endpoint)

    @property
    def object_endpoint(self) -> str:
        return urljoin(self.base_endpoint, self.project_id)

    def get_projects(self, readonly: bool = False) -> typing.List[ProjectOverview]:
        """Gets list of projects

        Args:
            readonly (bool, optional): Only archived projects. Defaults to False.

        Returns:
            json: List of all Projects
        """
        url = self.base_endpoint
        if readonly:
            url = f"{url}?readonly=true"
        response = self.get(url)
        return_data = list()
        for item in response.json()["results"]:
            return_data.append(ProjectOverview(item))
        return return_data

    def search(self, search_term: Optional[str] = "") -> typing.List[ProjectOverview]:
        """Searches projects by search term and retrieves all projects that match

        Args:
            search_term (Optional[str], optional): Search Term to look for. Defaults to None.

        Returns:
            typing.List[Project]: List of project overviews (without sections, findings) that match search
        """

        response = self.get(f"{self.base_endpoint}?search={search_term}")

        return_data = list()
        for item in response.json()["results"]:
            return_data.append(ProjectOverview(item))
        return return_data

    @cached_property
    def project(self) -> Project:
        return self._get_project()

    @cached_property
    def _project_dict(self) -> dict:
        url = self.object_endpoint
        return self.get(url).json()

    def _get_project(self) -> Project:
        return Project(
            self._project_dict,
            self.reptor.api.project_designs.project_design,
        )

    def export(self) -> bytes:
        """Exports a Project in archive format (tar.gz)"""
        url = urljoin(self.base_endpoint, f"{self.project_id}/export/all")
        return self.post(url).content

    def render(self) -> bytes:
        """Renders project to PDF"""
        # Get report checks
        checks = self.check_report(group_messages=True)
        for check, warnings in checks.items():
            if any([w.get("level") == "warning" for w in warnings]):
                self.log.warning(f'Report Check Warning: "{check}" (x{len(warnings)})')

        # Render report
        url = urljoin(self.base_endpoint, f"{self.project_id}/generate/")
        try:
            return self.post(url).content
        except HTTPError as e:
            try:
                for msg in e.response.json().get("messages", []):
                    if msg.get("level") == "error":
                        self.log.error(msg.get("message"))
                    elif msg.get("level") == "warning":
                        self.log.warning(msg.get("message"))
            except Exception:
                pass
            raise e

    def check_report(self, group_messages=False) -> dict:
        url = urljoin(self.base_endpoint, f"{self.project_id}/check")
        data = self.get(url).json()
        if group_messages:
            data = data.get("messages")
            # data is a list of dicts. group by "message" key
            grouped = dict()
            for item in data:
                grouped.setdefault(item["message"], []).append(item)
            return grouped
        return data

    def delete_project(self) -> None:
        url = self.object_endpoint
        self.delete(url)

    def duplicate_project(self) -> Project:
        """Duplicates Projects

        Returns:
            Project: Project Object
        """
        url = urljoin(self.base_endpoint, f"{self.project_id}/copy/")
        duplicated_project = self.post(url).json()
        return Project(
            duplicated_project,
            self.reptor.api.project_designs.project_design,
        )

    @contextmanager
    def duplicate_and_cleanup(self):
        original_project_id = self.project_id
        duplicated_project = self.duplicate_project()
        self.switch_project(duplicated_project.id)
        self.log.info(f"Duplicated project to {duplicated_project.id}")

        yield

        self.delete_project()
        self.switch_project(original_project_id)
        self.log.info(f"Cleaned up duplicated project")

    def switch_project(self, new_project_id) -> None:
        self.reptor._config._raw_config["project_id"] = new_project_id
        self._project_id = new_project_id
        self._init_attrs()
        self.reptor._api = None

    def get_sections(self) -> typing.List[Section]:
        """Gets all sections of a project

        Returns:
            typing.List[Section]: List of sections for this project
        """
        return_data = list()
        url = urljoin(self.base_endpoint, f"{self.project_id}/sections/")
        response = self.get(url).json()

        if not response:
            return return_data

        if not self.project_design:
            self.project_design = self.reptor.api.project_designs.project_design

        for item in response:
            section = Section(SectionRaw(item), self.project_design)
            return_data.append(section)
        return return_data

    def get_findings(self) -> typing.List[FindingRaw]:
        """Gets all findings of a project

        Returns:
            typing.List[FindingRaw]: List of findings for this project
        """
        url = urljoin(self.base_endpoint, f"{self.project_id}/findings/")
        response = self.get(url).json()

        if not response:
            return []
        return [FindingRaw(f) for f in response]

    def delete_finding(self, finding_id: str) -> None:
        url = urljoin(self.base_endpoint, f"{self.project_id}/findings/{finding_id}/")
        self.delete(url)

    def update_finding(self, finding_id: str, data: dict) -> FindingRaw:
        url = urljoin(self.base_endpoint, f"{self.project_id}/findings/{finding_id}/")
        return FindingRaw(self.patch(url, json=data).json())

    def create_finding(self, data: dict) -> FindingRaw:
        url = urljoin(self.base_endpoint, f"{self.project_id}/findings/")
        return FindingRaw(self.post(url, json=data).json())

    def create_finding_from_template(self, template_id: str) -> FindingRaw:
        url = urljoin(self.base_endpoint, f"{self.project_id}/findings/fromtemplate/")
        return FindingRaw(self.post(url, json={"template": template_id}).json())

    def update_section(self, section_id: str, data: dict) -> SectionRaw:
        url = urljoin(self.base_endpoint, f"{self.project_id}/sections/{section_id}/")
        return SectionRaw(self.patch(url, json=data).json())

    def update_project(self, data: dict) -> Project:
        url = urljoin(self.base_endpoint, f"{self.project_id}/")
        return Project(
            self.patch(url, json=data).json(),
            self.reptor.api.project_designs.project_design,
        )

    def update_project_design(self, design_id, force=False) -> Project:
        data = {
            "project_type": design_id,
            "force_change_project_type": True if force else False,
        }
        try:
            return self.update_project(data)
        except HTTPError as e:
            raise (HTTPError(e.response.text))

    def get_enabled_language_codes(
        self,
    ) -> list:  # TODO should not be in ProjectsAPI probably
        url = urljoin(self.reptor.get_config().get_server(), "api/v1/utils/settings/")
        settings = self.get(url).json()
        languages = [
            l["code"] for l in settings.get("languages", list()) if l["enabled"] == True
        ]
        return languages



import typing

import requests

import reptor.settings as settings
from reptor.lib.console import reptor_console


class APIClient:
    """Base API Client, holds all endpoint configuration and supplies subclasses with HTTP methods"""

    reptor: typing.Any
    base_endpoint: str
    endpoint: str
    item_id: str
    force_unlock: bool
    _project_id: str

    def __init__(self, require_project_id=True, **kwargs) -> None:
        self.reptor = kwargs.get("reptor", None)
        if not self.reptor:
            reptor_console.print(
                "[red]Make sure you access the API via the apimanager (reptor.api)[/red]"
            )
            exit(1)
        self.verify = not self.reptor.get_config().get("insecure", False)
        if not self.verify:
            import urllib3

            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)  # type: ignore

        try:
            self._project_id = self.reptor.get_active_project_id()
            self.debug(f"Project ID is {self._project_id}")
        except ValueError as e:
            if require_project_id:
                raise e
            self._project_id = ""

    @property
    def project_id(self) -> str:
        if not self._project_id:
            raise ValueError(
                "No Project ID configured. Try 'reptor conf' or use '--project-id'."
            )
        return self._project_id

    def _get_headers(self, json_content=True) -> typing.Dict:
        headers = dict()
        if json_content:
            headers["Content-Type"] = "application/json"
        headers["User-Agent"] = settings.USER_AGENT
        headers["Authorization"] = f"Bearer {self.reptor.get_config().get_token()}"
        headers_debug = headers.copy()
        headers_debug["Authorization"] = "[redacted]"
        self.debug(f"HTTP Headers: {headers_debug}")
        return headers

    def _prepare_kwargs(self, kwargs, json_content=True):
        headers = self._get_headers(json_content=json_content)
        if kwargs.get("headers"):
            kwargs["headers"].update(headers)
        else:
            kwargs["headers"] = headers
        if not kwargs.get("verify"):
            kwargs["verify"] = self.verify
        return kwargs

    @property
    def log(self):
        """Access the logging directly from plugin

        Returns:
            ReptorAdapter: Logging Module
        """
        return self.reptor.get_logger()

    @property
    def console(self):
        """Access the rich console that allows markdown etc.

        Returns:
            Console: rich console
        """
        return reptor_console

    def print(self, *args, **kwargs):
        print(*args, **kwargs)

    def success(self, msg, *args, **kwargs):
        """Use this to print Green text by default. You can change colors etc.

        See the logger.py for examples.

        Args:
            msg (str): Any message you want to print
        """
        self.log.success(msg, *args, **kwargs)

    def display(self, msg, *args, **kwargs):
        """Use this to print blue text by default. You can change colors etc.

        See the logger.py for examples.

        Args:
            msg (str): Any message you want to print
        """
        self.log.display(msg, *args, **kwargs)

    def highlight(self, msg, *args, **kwargs):
        """Prints a yellow message. Good for highlighting certain
        output.

        Args:
            msg (str): Any message you want to print
        """
        self.log.highlight(msg, *args, **kwargs)

    def debug(self, msg, *args, **kwargs):
        """Default DEBUG method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in DEBUG log
        """
        self.log.debug(msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        """Default INFO method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in INFO log
        """
        self.log.info(msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        """Default WARNING method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in WARNING log
        """
        self.log.warning(msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        """Default ERROR method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in ERROR log
        """
        self.log.error(msg, *args, **kwargs)

    def _do_request(
        self, url, method: str = "GET", json_content: bool = True, **kwargs
    ) -> requests.models.Response:
        methods = {
            "GET": requests.get,
            "POST": requests.post,
            "PUT": requests.put,
            "PATCH": requests.patch,
            "DELETE": requests.delete,
        }
        method = method.upper()
        if method not in methods.keys():
            raise ValueError(f"Method {method} not supported")
        self.debug(f"{method} URL: {url}")
        kwargs = self._prepare_kwargs(kwargs, json_content=json_content)
        response = methods[method](url, **kwargs)
        self.debug(f"Received response: {response.content[:1000]}")
        response.raise_for_status()
        return response

    def get(self, *args, **kwargs) -> requests.models.Response:
        return self._do_request(*args, method="GET", **kwargs)

    def post(self, *args, **kwargs) -> requests.models.Response:
        return self._do_request(*args, method="POST", **kwargs)

    def put(self, *args, **kwargs) -> requests.models.Response:
        return self._do_request(*args, method="PUT", **kwargs)

    def patch(self, *args, **kwargs) -> requests.models.Response:
        return self._do_request(*args, method="PATCH", **kwargs)

    def delete(self, *args, **kwargs) -> requests.models.Response:
        return self._do_request(*args, method="DELETE", **kwargs)



from reptor.lib.plugins.ToolBase import ToolBase
from reptor.models.Note import NoteTemplate


class Zap(ToolBase):
    """
    Parses ZAP generated reports in either XML or JSON

    Make sure you verify what type of findings you export.

    Recommended settings are to only export Medium and higher findings
    """

    meta = {
        "author": "Richard Schwabe",
        "name": "Zap",
        "version": "1.0",
        "license": "MIT",
        "tags": ["web", "zap"],
        "summary": "Parses ZAP reports (JSON, XML)",
    }
    risk_mapping = {
        "3": "🔴",
        "2": "🟠",
        "1": "🟡",
        "0": "🔵",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.notetitle = kwargs.get("notetitle") or "Zap"
        self.note_icon = "🌩️"
        if self.input_format == "raw":
            self.input_format = "json"

    def _parse_alert_data(self, data):
        return {
            "pluginid": data.find("pluginid").text,
            "alertRef": data.find("alertRef").text,
            "name": data.find("name").text,
            "riskcode": data.find("riskcode").text,
            "confidence": data.find("confidence").text,
            "riskdesc": data.find("riskdesc").text,
            "confidencedesc": data.find("confidencedesc").text,
            "desc": data.find("desc").text,
            "count": data.find("count").text,
            "solution": data.find("solution").text,
            "reference": data.find("reference").text,
            "cweid": data.find("cweid").text,
            "wascid": data.find("wascid").text,
            "sourceid": data.find("sourceid").text,
            "instances": [],
        }

    def _parse_instance_data(self, data):
        result = {
            "uri": data.find("uri").text,
            "method": data.find("method").text,
            "param": data.find("param").text,
            "attack": data.find("attack").text,
            "evidence": data.find("evidence").text,
            "otherinfo": data.find("otherinfo").text,
        }
        if data.find("requestheader"):
            result["requestheader"] = (data.find("requestheader").text,)
        if data.find("requestbody"):
            result["requestbody"] = (data.find("requestbody").text,)
        if data.find("responseheader"):
            result["responseheader"] = (data.find("responseheader").text,)
        return result

    def parse_xml(self):
        super().parse_xml(as_dict=False)
        return_data = list()
        for scan in self.xml_root:
            site = scan.attrib
            site["alerts"] = []

            for alert_item in scan[0]:
                alert = self._parse_alert_data(alert_item)

                for instance_item in alert_item.findall("./instances/instance"):
                    instance = self._parse_instance_data(instance_item)
                    alert["instances"].append(instance)

                site["alerts"].append(alert)

            return_data.append(site)

        self.parsed_input = return_data

    def parse_json(self):
        super().parse_json()
        parsed_input = list()
        for site in self.parsed_input.get("site", []):
            for attr in list(site.keys()):
                site[attr.strip("@")] = site.pop(attr)
            parsed_input.append(site)
        self.parsed_input = parsed_input

    def create_notes(self):
        data = dict()
        # Group by URL
        data = dict()
        for site in self.parsed_input:
            data[site["name"]] = [site]
        data = dict(
            sorted(data.items(), key=lambda x: len(x[1][0]["alerts"]), reverse=True)
        )

        # Create note structure
        ## Main note
        main_note = NoteTemplate()
        main_note.title = self.notetitle
        main_note.icon_emoji = self.note_icon

        ## Subnotes per site
        for url, site in data.items():
            site_note = NoteTemplate()
            site_note.title = f"{url} ({len(site[0]['alerts'])})"
            site_note.checked = False
            site_note.template = "site"
            site_note.template_data = site[0]
            # Subnotes per alert
            for alert in site[0]["alerts"]:
                alert_note = NoteTemplate()
                alert_note.title = (
                    f"{self.risk_mapping[alert['riskcode']]} {alert['name']}"
                )
                alert_note.checked = False
                alert_note.template = "alert"
                alert_note.template_data = alert
                site_note.children.append(alert_note)
            main_note.children.append(site_note)
        return main_note


loader = Zap



import json
import os
from pathlib import Path
from unittest.mock import MagicMock, Mock

import pytest
from django.template.loader import render_to_string

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin
from reptor.models.Finding import FindingRaw
from reptor.models.Project import Project
from reptor.models.ProjectDesign import ProjectDesign

from ..Sslyze import Sslyze


class TestSslyze(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        Sslyze.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.sslyze = Sslyze(reptor=self.reptor)

    def _load_json_data(self):
        self.sslyze.input_format = "json"
        filepath = os.path.join(os.path.dirname(__file__), "./data/sslyze.json")
        with open(filepath, "r") as f:
            self.sslyze.raw_input = f.read()

    def test_finding_content(self):
        context = {
            "data": [
                {
                    "hostname": "example.com",
                    "port": "443",
                    "ip_address": "127.0.0.1",
                    "vulnerabilities": {"heartbleed": True},
                    "protocols": {
                        "sslv2": {"a": "b"},
                    },
                    "certinfo": {
                        "certificate_matches_hostname": False,
                        "certificate_untrusted": ["Android", "iOS"],
                    },
                    "misconfigurations": {
                        "compression": False,
                        "downgrade": True,
                        "no_secure_renegotiation": True,
                        "accepts_client_renegotiation": False,
                    },
                    "has_vulnerabilities": True,
                    "flag_for_finding": True,
                    "has_insecure_protocols": True,
                    "has_weak_ciphers": True,
                    "has_weak_protocols": True,
                    "has_insecure_ciphers": True,
                    "has_cert_issues": True,
                    "has_misconfigurations": True,
                },
                {
                    "hostname": "www.example.com",
                    "port": "443",
                    "ip_address": "127.0.0.1",
                    "vulnerabilities": {"robot": True},
                    "protocols": {
                        "sslv3": {"a": "b"},
                        "tlsv1_1": {"weak_ciphers": []},
                    },
                    "certinfo": {
                        "certificate_matches_hostname": True,
                        "certificate_untrusted": ["Android"],
                    },
                    "misconfigurations": {
                        "compression": True,
                        "downgrade": True,
                        "no_secure_renegotiation": True,
                        "accepts_client_renegotiation": True,
                    },
                    "has_vulnerabilities": True,
                    "flag_for_finding": True,
                    "has_insecure_protocols": True,
                    "has_weak_ciphers": True,
                    "has_weak_protocols": True,
                    "has_insecure_ciphers": False,
                    "has_cert_issues": True,
                    "has_misconfigurations": True,
                },
                {
                    "hostname": "ftp.example.com",
                    "port": "443",
                    "ip_address": "127.0.0.1",
                    "vulnerabilities": {"openssl_ccs": True},
                    "protocols": {"tlsv1_2": {"weak_ciphers": []}},
                    "misconfigurations": {
                        "compression": False,
                        "downgrade": False,
                        "no_secure_renegotiation": False,
                        "accepts_client_renegotiation": False,
                    },
                    "certinfo": {
                        "certificate_matches_hostname": False,
                        "certificate_untrusted": [],
                    },
                    "has_vulnerabilities": True,
                    "flag_for_finding": True,
                    "has_insecure_protocols": True,
                    "has_insecure_ciphers": True,
                    "has_weak_ciphers": True,
                    "has_weak_protocols": True,
                    "has_cert_issues": True,
                    "has_misconfigurations": False,
                },
            ],
        }

        # Patch API
        self.reptor.api.templates.search = Mock(return_value=[])

        self._load_json_data()
        self.sslyze.parsed_input = None
        self.sslyze.parse()
        data = self.sslyze.preprocess_for_template()
        assert isinstance(data, dict)
        data.update(context)

        # Patch preprocess_for_template
        self.sslyze.preprocess_for_template = Mock(return_value=data)
        self.sslyze.generate_findings()
        assert len(self.sslyze.findings) == 1

        summary = "We found that example.com:443 and 2 other services had a weak TLS setup. This might impact the confidentiality and integrity of your data in transit."

        assert summary in self.sslyze.findings[0].data.summary.value

        certinfo_data = """The certificates of example.com:443 and 2 other services are untrusted by common browsers:

* example.com:443 (unmatching hostname; untrusted by Android, iOS)
* www.example.com:443 (untrusted by Android)
* ftp.example.com:443 (unmatching hostname)"""
        assert certinfo_data in self.sslyze.findings[0].data.description.value

        misconfigurations_data = """Additionally, we detected the following misconfigurations:

|  | TLS Compression | Downgrade (no SCSV fallback) | No Secure Renegotiation | Client Renegotiation |
| ------- | ------- | ------- | ------- | ------- |
| example.com:443 |  <span style="color: green">No</span> | <span style="color: red">Yes</span> | <span style="color: red">Yes</span> | <span style="color: green">No</span> |
| www.example.com:443 |  <span style="color: red">Yes</span> | <span style="color: red">Yes</span> | <span style="color: red">Yes</span> | <span style="color: red">Yes</span> |
| ftp.example.com:443 |  <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: green">No</span> |"""
        assert misconfigurations_data in self.sslyze.findings[0].data.description.value

        vulnerabilities_data = """example.com:443 and 2 other services used outdated TLS libraries. This results in the following vulnerabilities:


|  | Heartbleed | Robot Attack | OpenSSL CCS |
| ------- | ------- | ------- | ------- |
| example.com:443 |  <span style="color: red">Vulnerable</span> | <span style="color: green">OK</span> | <span style="color: green">OK</span> |
| www.example.com:443 |  <span style="color: green">OK</span> | <span style="color: red">Vulnerable</span> | <span style="color: green">OK</span> |
| ftp.example.com:443 |  <span style="color: green">OK</span> | <span style="color: green">OK</span> | <span style="color: red">Vulnerable</span> |"""
        assert vulnerabilities_data in self.sslyze.findings[0].data.description.value

        protocols_data = """We also found out that example.com:443 and 2 other services had insecure ciphers or protocols enabled:

|  | SSLv2 | SSLv3 | TLS 1.0 | TLS 1.1 | Weak Ciphers | Insecure Ciphers |
| ------- | ------- | ------- | ------- | ------- | ------- | ------- |
| example.com:443 |  <span style="color: red">Yes</span> | <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: orange">Yes</span> | <span style="color: red">Yes</span> |
| www.example.com:443 |  <span style="color: green">No</span> | <span style="color: red">Yes</span> | <span style="color: green">No</span> | <span style="color: orange">Yes</span> | <span style="color: orange">Yes</span> | <span style="color: green">No</span> |
| ftp.example.com:443 |  <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: orange">Yes</span> | <span style="color: red">Yes</span> |"""
        assert protocols_data in self.sslyze.findings[0].data.description.value

    def test_generate_and_push_findings(self):
        # Patch API
        self.reptor.api.templates.search = Mock(return_value=[])

        self._load_json_data()
        # Assert "create_finding" is called if no findings exist
        self.sslyze.reptor.api.projects.get_findings = Mock(return_value=[])
        self.reptor.api.projects.create_finding = MagicMock()

        self.sslyze.generate_and_push_findings()
        assert self.reptor.api.projects.create_finding.called

        # Assert "create_finding" is not called if finding with same title exists
        finding_raw = FindingRaw(
            {"data": {"title": "Weak TLS setup might impact encryption"}}
        )
        self.reptor.api.projects._project_dict = {"project_type": ""}
        self.reptor.api.project_designs.project_design = ProjectDesign()
        self.reptor.api.projects.get_findings = Mock(return_value=[finding_raw])
        self.reptor.api.projects.create_finding = MagicMock()

        self.sslyze.generate_and_push_findings()
        assert not self.sslyze.reptor.api.projects.create_finding.called

    def test_generate_findings(self):
        # Patch API
        self.reptor.api.templates.search = Mock(return_value=[])

        self._load_json_data()
        self.sslyze.generate_findings()
        assert len(self.sslyze.findings) == 1
        fndg = self.sslyze.findings[0]
        assert fndg.data.title.value == "Weak TLS setup might impact encryption"
        assert (
            "had insecure ciphers or protocols enabled" in fndg.data.description.value
        )
        assert (
            fndg.data.affected_components.value[0].value
            == "www.example.com:443 (127.0.0.1)"
        )
        assert [r.value for r in fndg.data.references.value] == [
            "https://ssl-config.mozilla.org/",
            "https://ciphersuite.info/",
        ]

    def test_parse(self):
        result_dict = {"a": "b"}
        self.sslyze.raw_input = json.dumps(result_dict)
        self.sslyze.parse()
        assert self.sslyze.parsed_input == result_dict

    def test_preprocess_for_template(self):
        result = {
            "data": [
                {
                    "hostname": "www.example.com",
                    "port": "443",
                    "ip_address": "127.0.0.1",
                    "protocols": {
                        "sslv2": {"weak_ciphers": [], "insecure_ciphers": []},
                        "tlsv1_2": {
                            "insecure_ciphers": [],
                            "weak_ciphers": [
                                "DHE-RSA-AES256-SHA256",
                                "AES256-SHA256",
                                "ECDHE-RSA-AES256-SHA384",
                                "DHE-RSA-AES256-SHA",
                                "AES256-SHA",
                                "AES256-GCM-SHA384",
                                "DHE-RSA-AES128-SHA256",
                                "DHE-RSA-AES128-SHA",
                                "AES128-SHA",
                                "AES128-SHA256",
                                "AES128-GCM-SHA256",
                                "ECDHE-RSA-AES128-SHA256",
                            ],
                        },
                    },
                    "has_insecure_protocols": True,
                    "has_weak_protocols": False,
                    "has_weak_ciphers": True,
                    "has_insecure_ciphers": False,
                    "certinfo": {
                        "certificate_matches_hostname": True,
                        "has_sha1_in_certificate_chain": False,
                        "certificate_untrusted": ["Android"],
                    },
                    "has_cert_issues": True,
                    "vulnerabilities": {
                        "heartbleed": True,
                        "openssl_ccs": True,
                        "robot": False,
                    },
                    "has_vulnerabilities": True,
                    "misconfigurations": {
                        "compression": False,
                        "downgrade": True,
                        "client_renegotiation": False,
                        "no_secure_renegotiation": True,
                    },
                    "has_misconfigurations": True,
                    "flag_for_finding": True,
                    "has_insecure_protocols": True,
                    "has_insecure_ciphers": False,
                    "has_cert_issues": True,
                    "has_misconfigurations": True,
                }
            ]
        }
        self._load_json_data()
        self.sslyze.parsed_input = None
        self.sslyze.parse()
        data = self.sslyze.preprocess_for_template()
        assert data == result

    def test_format(self):
        # result = """| Host | Port | Service | Version |"""
        self._load_json_data()

        # Protocols template
        protocols_result = """# Sslyze

## 🚩 www.example.com:443 (127.0.0.1)


**Protocols**

 * <span style="color: red">SSLv2</span>
 * <span style="color: green">TLS 1.2</span>


**Certificate Information**

 * <span style="color: red">Certificate untrusted by:</span>
    * Android
 * Certificate matches hostname: <span style="color: green">Yes</span>
 * SHA1 in certificate chain: <span style="color: green">No</span>


**Vulnerabilities**

 * Heartbleed: <span style="color: red">Yes</span>
 * Robot Attack: <span style="color: green">No</span>
 * OpenSSL CCS (CVE-2014-0224): <span style="color: red">Yes</span>


**Misconfigurations**

 * Compression: <span style="color: green">No</span>
 * Downgrade Attack (no SCSV fallback): <span style="color: red">Yes</span>
 * No Secure Renegotiation: <span style="color: red">Yes</span>
 * Client Renegotiation: <span style="color: green">No</span>


**Weak Cipher Suites**

 * <span style="color: orange">DHE-RSA-AES256-SHA256</span> (TLS 1.2)
 * <span style="color: orange">AES256-SHA256</span> (TLS 1.2)
 * <span style="color: orange">ECDHE-RSA-AES256-SHA384</span> (TLS 1.2)
 * <span style="color: orange">DHE-RSA-AES256-SHA</span> (TLS 1.2)
 * <span style="color: orange">AES256-SHA</span> (TLS 1.2)
 * <span style="color: orange">AES256-GCM-SHA384</span> (TLS 1.2)
 * <span style="color: orange">DHE-RSA-AES128-SHA256</span> (TLS 1.2)
 * <span style="color: orange">DHE-RSA-AES128-SHA</span> (TLS 1.2)
 * <span style="color: orange">AES128-SHA</span> (TLS 1.2)
 * <span style="color: orange">AES128-SHA256</span> (TLS 1.2)
 * <span style="color: orange">AES128-GCM-SHA256</span> (TLS 1.2)
 * <span style="color: orange">ECDHE-RSA-AES128-SHA256</span> (TLS 1.2)"""
        self.sslyze.format()
        assert self.sslyze.formatted_input.strip() == protocols_result.strip()

    def test_path_methods(self):
        plugin_path = Path(os.path.normpath(Path(os.path.dirname(__file__)) / ".."))
        template_paths = self.sslyze.get_plugin_dir_paths(
            plugin_path, "templates", skip_user_plugins=True
        )
        assert len(template_paths) == 1
        assert isinstance(template_paths[0], Path)
        assert template_paths[0] == plugin_path / "templates"

        templates = self.sslyze.get_filenames_from_paths(template_paths, "md")
        template_names = [
            "certinfo",
            "summary",
            "misconfigurations",
            "protocols",
            "vulnerabilities",
            "weak_ciphers",
        ]
        assert all([t in templates for t in template_names])



import json

import pytest

from reptor.models.Finding import Finding
from reptor.models.Project import Project, ProjectOverview
from reptor.models.ProjectDesign import ProjectDesign
from reptor.models.Section import Section
from reptor.models.User import User
from reptor.settings import DEFAULT_PROJECT_DESIGN


class TestProjectModelParsing:
    example_project = """
    {
            "id": "4cf78324-8502-4fb0-936a-724892d3c539",
            "created": "2023-06-16T19:12:00.478314Z",
            "updated": "2023-06-25T13:49:48.935500Z",
            "name": "PNPT Exam",
            "project_type": "fa670018-e6ef-4b73-989b-1e4c4af09cee",
            "language": "en-US",
            "tags": [],
            "readonly": true,
            "source": "created",
            "copy_of": null,
            "members": [
                {
                    "id": "f2c9bad4-c916-4c18-9f76-d5ef94b34453",
                    "username": "reptor",
                    "name": "Bjoern Schwabe",
                    "title_before": "",
                    "first_name": "Bjoern",
                    "middle_name": null,
                    "last_name": "Schwabe",
                    "title_after": "",
                    "is_active": true,
                    "roles": [
                        "pentester"
                    ]
                },
                {
                    "id": "f2c9bad4-c916-4c18-9f76-d5ef94b34454",
                    "username": "richard",
                    "name": "Richard Schwabe",
                    "title_before": "",
                    "first_name": "Richard",
                    "middle_name": null,
                    "last_name": "Schwabe",
                    "title_after": "",
                    "is_active": true,
                    "roles": [
                        "pentester"
                    ]
                }
            ],
            "imported_members": [],
            "details": "http://localhost:8000/api/v1/pentestprojects/4cf78324-8502-4fb0-936a-724892d3c539",
            "findings": [{"id": "a9e6fd6b-773f-46a8-8a5c-58fdeefbdfc0", "created": "2023-08-25T11:57:03.886931Z", "updated": "2023-08-25T11:57:03.889063Z", "project": "4820bd5d-51f1-4dca-a4a4-78ba935b615c", "project_type": "50dfad6e-805d-4215-b098-a73e03b1ec3b", "language": "en-US", "lock_info": null, "template": null, "assignee": null, "status": "in-progress", "order": 1, "data": {"cvss": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:L/I:L/A:N", "title": "Reflected Cross-Site Scripting (XSS)", "impact": "", "summary": "We detected a reflected XSS vulnerability.\\n\\n**XSS targets**\\n\\n* https://example.com/alert(1)\\n* https://example.com/q=alert(1)\\n", "severity": null, "references": ["https://owasp.org/www-community/attacks/xss/"], "description": "", "precondition": "", "retest_notes": "", "retest_status": null, "wstg_category": null, "recommendation": "HTML encode user-supplied inputs.", "owasp_top10_2021": null, "affected_components": ["https://example.com/alert(1)", "https://example.com/q=alert(1)"], "short_recommendation": ""}}],
            "sections": [{"id": "other", "label": "Executive Summary", "fields": ["title"], "project": "4820bd5d-51f1-4dca-a4a4-78ba935b615c", "project_type": "50dfad6e-805d-4215-b098-a73e03b1ec3b", "language": "en-US", "lock_info": null, "assignee": null, "status": "in-progress", "data": {"title": "Test"}}],
            "notes": "http://localhost:8000/api/v1/pentestprojects/4cf78324-8502-4fb0-936a-724892d3c539/notes",
            "images": "http://localhost:8000/api/v1/pentestprojects/4cf78324-8502-4fb0-936a-724892d3c539/images"
        }"""
    example_project_overview = """{
        "id": "4820bd5d-51f1-4dca-a4a4-78ba935b615c",
        "created": "2023-08-22T09:22:26.874121Z",
        "updated": "2023-09-08T06:25:15.574675Z",
        "name": "Project Funghi",
        "project_type": "5755c339-9ca7-4f83-a251-5490cfcf67e7",
        "language": "en-US",
        "tags": [],
        "readonly": false,
        "source": "created",
        "copy_of": null,
        "override_finding_order": false,
        "members": [],
        "imported_members": [],
        "details": "https://syslifters.sysre.pt/api/v1/pentestprojects/4820bd5d-51f1-4dca-a4a4-78ba935b615c",
        "findings": "https://syslifters.sysre.pt/api/v1/pentestprojects/4820bd5d-51f1-4dca-a4a4-78ba935b615c/findings",
        "sections": "https://syslifters.sysre.pt/api/v1/pentestprojects/4820bd5d-51f1-4dca-a4a4-78ba935b615c/sections",
        "notes": "https://syslifters.sysre.pt/api/v1/pentestprojects/4820bd5d-51f1-4dca-a4a4-78ba935b615c/notes",
        "images": "https://syslifters.sysre.pt/api/v1/pentestprojects/4820bd5d-51f1-4dca-a4a4-78ba935b615c/images"
    }"""

    def test_project_custom_fields(self):
        project_with_custom_fields = """{
    "id": "c69d5b9f-3964-490b-9831-540d87aa97b2",
    "created": "2023-09-22T11:49:49.930799Z",
    "updated": "2023-10-16T06:20:34.202981Z",
    "name": "Project Funghi",
    "project_type": "6e9cf824-7b01-4551-8c24-55f2e342b87a",
    "language": "en-US",
    "tags": [],
    "readonly": false,
    "source": "imported",
    "copy_of": null,
    "override_finding_order": false,
    "members": [{
            "id": "ed4196c7-f60a-48bf-8119-dc1642946231",
            "username": "admin",
            "name": "",
            "title_before": null,
            "first_name": "",
            "middle_name": null,
            "last_name": "",
            "title_after": null,
            "is_active": true,
            "roles": []
        }
    ],
    "imported_members": [],
    "findings": [{
            "id": "eade9987-7029-453a-ad28-ed028a7740d7",
            "created": "2023-10-13T06:17:28.772372Z",
            "updated": "2023-10-16T06:26:15.831429Z",
            "project": "c69d5b9f-3964-490b-9831-540d87aa97b2",
            "project_type": "6e9cf824-7b01-4551-8c24-55f2e342b87a",
            "language": "en-US",
            "lock_info": {},
            "template": null,
            "assignee": null,
            "status": "in-progress",
            "order": 2,
            "data": {
                "title": "My Title",
                "target": "My custom field (target)"
            }
        }
    ],
    "sections": [{
            "id": "executive_summary",
            "label": "Executive Summary",
            "fields": ["executive_summary", "user"],
            "project": "c69d5b9f-3964-490b-9831-540d87aa97b2",
            "project_type": "6e9cf824-7b01-4551-8c24-55f2e342b87a",
            "language": "en-US",
            "lock_info": null,
            "assignee": null,
            "status": "in-progress",
            "data": {
                "executive_summary": "My Executive Summary"
            }
        }
    ]
}"""
        project_design = DEFAULT_PROJECT_DESIGN
        DEFAULT_PROJECT_DESIGN["finding_fields"]["target"] = {
            "type": "string",
            "label": "Target",
            "origin": "custom",
            "default": "TODO: Target",
            "required": True,
            "spellcheck": True,
        }
        DEFAULT_PROJECT_DESIGN["report_sections"].append(
            {
                "id": "executive_summary",
                "label": "Executive Summary",
                "fields": ["executive_summary"],
            }
        )
        DEFAULT_PROJECT_DESIGN["report_fields"]["executive_summary"] = {
            "type": "markdown",
            "label": "Executive Summary",
            "origin": "custom",
            "default": "TODO: Executive Summary",
            "required": True,
        }
        api_test_data = json.loads(project_with_custom_fields)

        project_design = ProjectDesign(project_design)
        test_project = Project(api_test_data, project_design)
        assert test_project.findings[0].data.target.value == "My custom field (target)"  # type: ignore
        assert (
            test_project.sections[0].data.executive_summary.value  # type: ignore
            == "My Executive Summary"
        )

    def test_project_overview_parsing(self):
        api_test_data = json.loads(self.example_project_overview)
        with pytest.raises(ValueError):
            Project(api_test_data, ProjectDesign())

        test_project = ProjectOverview(api_test_data)
        assert test_project.id == "4820bd5d-51f1-4dca-a4a4-78ba935b615c"
        assert test_project.name == "Project Funghi"
        assert (
            test_project.sections
            == "https://syslifters.sysre.pt/api/v1/pentestprojects/4820bd5d-51f1-4dca-a4a4-78ba935b615c/sections"
        )
        assert (
            test_project.findings
            == "https://syslifters.sysre.pt/api/v1/pentestprojects/4820bd5d-51f1-4dca-a4a4-78ba935b615c/findings"
        )

    def test_project_parsing(self):
        api_test_data = json.loads(self.example_project)

        test_project = Project(api_test_data, ProjectDesign())
        assert test_project.id == "4cf78324-8502-4fb0-936a-724892d3c539"
        assert test_project.members[0].id == "f2c9bad4-c916-4c18-9f76-d5ef94b34453"
        assert test_project.members[1].username == "richard"
        assert test_project.project_type == "fa670018-e6ef-4b73-989b-1e4c4af09cee"

        assert isinstance(test_project.sections[0], Section)
        assert isinstance(test_project.findings[0], Finding)
        assert isinstance(test_project.members[0], User)

        test_project_dict = test_project.to_dict()
        # Ensure items are still as before to_dict
        assert isinstance(test_project.sections[0], Section)
        assert isinstance(test_project.findings[0], Finding)
        assert isinstance(test_project.members[0], User)

        assert isinstance(test_project_dict["sections"], list)
        assert isinstance(test_project_dict["findings"], list)
        assert isinstance(test_project_dict["members"], list)

        assert (
            test_project.findings[0].data.title.value
            == "Reflected Cross-Site Scripting (XSS)"
        )
        assert test_project.sections[0].data.title.value == "Test"  # type: ignore

        assert (
            test_project_dict["findings"][0]["data"]["title"]
            == "Reflected Cross-Site Scripting (XSS)"
        )
        assert test_project_dict["sections"][0]["data"]["title"] == "Test"



import io
import json
import tarfile
import typing

import yaml

from reptor.lib.plugins.Base import Base
from reptor.utils.table import make_table


class Template(Base):
    """
    Work with finding templates.
    """

    meta = {
        "name": "Template",
        "summary": "Queries Finding Templates from SysReptor",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.arg_search = kwargs.get("search")
        self.export: typing.Optional[str] = kwargs.get("export")
        self.language: typing.Optional[str] = kwargs.get("language")
        self.output: typing.Optional[str] = kwargs.get("output")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        templates_parsers = parser.add_argument_group()
        templates_parsers.add_argument(
            "--search", help="Search for term", action="store", default=None
        )
        templates_parsers.add_argument(
            "--language",
            help='Template language for export format "plain", e.g. "en"',
            action="store",
            default=None,
        )
        templates_parsers.add_argument(
            "--export",
            help="Export templates",
            choices=["tar.gz", "json", "yaml", "plain"],
            type=str.lower,
            action="store",
            dest="export",
            default=None,
        )
        parser.add_argument(
            "-o",
            "--output",
            metavar="FILENAME",
            help="Filename for output",
            action="store",
            default=None,
        )

    def _merge_tars(self, tars: typing.Iterable) -> bytes:
        result_io = io.BytesIO()
        with tarfile.open(fileobj=result_io, mode="w:gz") as result_tar:
            for tar in tars:
                tar = io.BytesIO(tar)
                with tarfile.open(fileobj=tar, mode="r:gz") as tar_file:
                    for member in tar_file.getmembers():
                        if member.isdir():
                            result_tar.addfile(member)
                        else:
                            result_tar.addfile(member, tar_file.extractfile(member))
        result_io.seek(0)
        return result_io.read()

    def export_archive(
        self,
        template_ids: typing.List[str],
        filename: str,
        stdout: bool = False,
    ):
        tar = self._merge_tars(
            ((self.reptor.api.templates.export(id) for id in template_ids))
        )
        self.deliver_file(
            content=tar,
            filename=filename,
            upload=False,
            stdout=stdout,
        )
        return

    def export_templates(
        self,
        template_ids: typing.List[str],
        format: typing.Optional[str] = "json",
        language: typing.Optional[str] = None,
        filename: typing.Optional[str] = None,
        stdout: bool = True,
    ):
        if format == "plain" and not stdout:
            raise ValueError('"plain" can only be used with stdout')
        templates = list()
        output = ""
        for template_id in template_ids:
            templates.append(self.reptor.api.templates.get_template(template_id))
        if format == "json":
            output = json.dumps([t.to_dict() for t in templates], indent=2)
        elif format == "yaml":
            output = yaml.dump([t.to_dict() for t in templates])
        elif format == "plain":
            for template in templates:
                for i, translation in enumerate(template.translations):
                    if i != 0:
                        self.print("")
                    if language and not translation.language.lower().startswith(
                        language
                    ):
                        continue
                    translation_data = translation.data.to_dict()
                    self.console.print(
                        f"[bold][red]{translation_data.pop('title')} ({translation.language})[/red][/bold]"
                    )
                    for key, value in translation_data.items():
                        if value:
                            if isinstance(value, list):
                                if len(value) > 1:
                                    value = "\n".join(
                                        ["* " + item.__str__() for item in value]
                                    )
                                elif len(value) == 1:
                                    value = value[0]
                            self.console.print(
                                f"[bold][blue]{key} ({translation.language})[/blue][/bold]"
                            )
                            self.print(value.__str__())
                            self.print("")
            return

        self.deliver_file(
            content=output.encode(),
            filename=filename or "",
            upload=False,
            stdout=stdout,
        )

    def run(self):
        filename = self.output

        if not self.arg_search:
            templates = self.reptor.api.templates.get_template_overview()
        else:
            templates = self.reptor.api.templates.search(self.arg_search)

        if not templates:
            self.display("Did not find any finding templates.")
            return

        if self.export == "tar.gz":
            stdout = False
            if self.output == "-":
                stdout = True
            if len(templates) == 1 and not self.output:
                filename = (
                    f"{templates[0].translations[0].data.title or 'template'}.tar.gz"
                )
            else:
                filename = "templates.tar.gz"
            self.export_archive(
                [t.id for t in templates], filename=filename, stdout=stdout
            )
        elif self.export:
            stdout = True
            if self.output == "-" or self.output is None:
                filename = None
            elif self.output:
                stdout = False
                filename = self.output
            self.export_templates(
                [t.id for t in templates],
                format=self.export,
                language=self.language,
                filename=filename,
                stdout=stdout,
            )
        else:
            table = make_table(["Title", "Usage Count", "Tags", "Translations", "ID"])
            for template in templates:
                main_translation = [t for t in template.translations if t.is_main][0]
                table.add_row(
                    main_translation.data.title,
                    str(template.usage_count),
                    ",".join(template.tags),
                    ",".join(t.language for t in template.translations),
                    template.id,
                )
            self.console.print(table)


loader = Template



import os
from pathlib import Path

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin
from reptor.models.Note import NoteTemplate

from ..Nmap import Nmap


class TestNmap(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        Nmap.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.nmap = Nmap(reptor=self.reptor, template=Nmap.template)

    def _load_grepable_data(self):
        self.nmap.input_format = "grepable"
        filepath = os.path.join(os.path.dirname(__file__), "./data/grepable.txt")
        with open(filepath, "r") as f:
            self.nmap.raw_input = f.read()

    def _load_xml_data(self, xml_file):
        self.nmap.input_format = "xml"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{xml_file}")
        with open(filepath, "r") as f:
            self.nmap.raw_input = f.read()

    def test_grepable_parse(self):
        result_dict = [
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": "nginx (reverse proxy)",
            },
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "443",
                "protocol": "tcp",
                "service": "ssl/http",
                "version": "nginx (reverse proxy)",
            },
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "8080",
                "protocol": "tcp",
                "service": "",
                "version": "",
            },
        ]
        self._load_grepable_data()
        self.nmap.parse()
        assert self.nmap.parsed_input == result_dict

    def test_xml_parse_multi_target(self):
        entries = [
            {
                "ip": "142.250.180.228",
                "hostname": "www.google.com",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": "gws",
            },
            {
                "ip": "142.250.180.228",
                "hostname": "www.google.com",
                "port": "443",
                "protocol": "tcp",
                "service": "https",
                "version": "gws",
            },
            {
                "ip": "34.249.200.254",
                "hostname": "www.syslifters.com",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": None,
            },
            {
                "ip": "34.249.200.254",
                "hostname": "www.syslifters.com",
                "port": "443",
                "protocol": "tcp",
                "service": "https",
                "version": None,
            },
        ]
        self._load_xml_data("nmap_multi_target.xml")
        self.nmap.parse()
        for entry in entries:
            assert entry in self.nmap.parsed_input

    def test_xml_parse_single_target(self):
        entries = [
            {
                "ip": "63.35.51.142",
                "hostname": "www.syslifters.com",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": None,
            }
        ]
        self._load_xml_data("nmap_single_target_single_port.xml")
        self.nmap.parse()
        for entry in entries:
            assert entry in self.nmap.parsed_input

    def test_xml_parse_without_hostname(self):
        entries = [
            {
                "ip": "142.251.208.164",
                "hostname": None,
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": None,
            },
            {
                "ip": "142.251.208.164",
                "hostname": None,
                "port": "443",
                "protocol": "tcp",
                "service": "https",
                "version": None,
            },
        ]
        self._load_xml_data("nmap_without_hostname.xml")
        self.nmap.parse()
        for entry in entries:
            assert entry in self.nmap.parsed_input

    def test_process_parsed_input(self):
        # test without hostname
        self.nmap.parsed_input = [
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": "nginx (reverse proxy)",
            },
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "443",
                "protocol": "tcp",
                "service": "ssl/http",
                "version": "nginx (reverse proxy)",
            },
        ]

        data = self.nmap.preprocess_for_template()
        assert data == {"data": self.nmap.parsed_input, "show_hostname": False}

        # test with hostname
        self.nmap.parsed_input = [
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": "nginx (reverse proxy)",
            },
            {
                "ip": "127.0.0.1",
                "hostname": "localhost",
                "port": "443",
                "protocol": "tcp",
                "service": "ssl/http",
                "version": "nginx (reverse proxy)",
            },
        ]

        data = self.nmap.preprocess_for_template()
        assert data == {"data": self.nmap.parsed_input, "show_hostname": True}

    def test_format_nmap(self):
        result = """# Nmap

| Host | Port | Service | Version |
| ------- | ------- | ------- | ------- |
| 127.0.0.1 | 80/tcp | http | nginx (reverse proxy) |
| 127.0.0.1 | 443/tcp | ssl/http | nginx (reverse proxy) |
| 127.0.0.1 | 8080/tcp | n/a | n/a |

## 127.0.0.1

| Host | Port | Service | Version |
| ------- | ------- | ------- | ------- |
| 127.0.0.1 | 80/tcp | http | nginx (reverse proxy) |
| 127.0.0.1 | 443/tcp | ssl/http | nginx (reverse proxy) |
| 127.0.0.1 | 8080/tcp | n/a | n/a |
"""
        self._load_grepable_data()
        self.nmap.format()
        assert isinstance(self.nmap.formatted_input, str)
        assert self.nmap.formatted_input.strip() == result.strip()

        result = """# Nmap

| Hostname | IP | Port | Service | Version |
| ------- | ------- | ------- | ------- | ------- |
| www.syslifters.com | 63.35.51.142 | 80/tcp | http | n/a |

## 63.35.51.142

| Hostname | IP | Port | Service | Version |
| ------- | ------- | ------- | ------- | ------- |
| www.syslifters.com | 63.35.51.142 | 80/tcp | http | n/a |"""
        self.nmap.parsed_input = None
        self._load_xml_data("nmap_single_target_single_port.xml")
        self.nmap.format()
        assert isinstance(self.nmap.formatted_input, str)
        assert self.nmap.formatted_input.strip() == result.strip()



import json
import subprocess

import pytest

from reptor.plugins.core.Conf.tests.conftest import projects_api, get_note


@pytest.mark.integration
class TestIntegrationProjectFinding(object):
    def test_render_project(self, projects_api):
        p = subprocess.Popen(
            ["reptor", "projectfindings", "--format", "json"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        assert p.returncode == 0
        json.loads(out)

        p = subprocess.Popen(
            ["reptor", "projectfindings", "--format", "json", "--upload"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        assert p.returncode == 0

        note = get_note("Uploads", None)
        note_last_line = note["text"].splitlines()[-1]  # type: ignore
        assert f"[{projects_api.project.name}.json]" in note_last_line



from unittest.mock import MagicMock

import pytest
from requests.exceptions import HTTPError

from reptor.lib.exceptions import LockedException
from reptor.lib.reptor import Reptor
from reptor.models.Note import Note

from ..NotesAPI import NotesAPI


class TestNotesAPI:
    class MockResponse:
        def __init__(self, content, status_code, raise_exception=False):
            self.content = content
            self.status_code = status_code
            self.raise_exception = raise_exception

        def raise_for_status(self):
            if self.raise_exception:
                raise HTTPError("Mocked HTTPError")
            return

        def json(self):
            return self.content

    test_note = {
        "id": "51abd3d3-803e-43a5-aa25-bf30b7fbf70a",
        "created": "2023-09-07T14:00:32.777492Z",
        "updated": "2023-09-07T14:00:45.409725Z",
        "lock_info": None,
        "title": "My Note",
        "text": "My Content",
        "checked": None,
        "icon_emoji": "\ud83d\udd25",
        "status_emoji": None,
        "order": 4,
        "parent": "8880ce39-90ed-4383-9320-d5d74b1ae34f",
    }
    locked_note = {
        "id": "1c76a78f-472c-445f-993a-214b86a05a49",
        "created": "2023-09-06T19:52:55.772872Z",
        "updated": "2023-09-06T19:52:56.475750Z",
        "lock_info": {
            "created": "2023-09-07T16:52:11.215893Z",
            "updated": "2023-09-07T16:52:11.216099Z",
            "last_ping": "2023-09-07T16:52:11.215905Z",
            "expires": "2023-09-07T16:53:41.215905Z",
            "user": {
                "id": "f18706c8-bff2-4526-b462-48d6c0f94d92",
                "username": "timmi",
                "name": "",
                "title_before": None,
                "first_name": "",
                "middle_name": None,
                "last_name": "",
                "title_after": None,
                "is_active": True,
            },
        },
        "title": "My Note",
        "text": "[2023-09-06 21:52:56]\n*Upload me*",
        "checked": None,
        "icon_emoji": None,
        "status_emoji": None,
        "order": 1,
        "parent": "4a071c2a-32fc-40bc-8157-54b4e944541f",
        "assignee": None,
    }

    @pytest.fixture(autouse=True)
    def setUp(self):
        self.reptor = Reptor()
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["cli"] = {"private_note": True}
        self.notes = NotesAPI(reptor=self.reptor)

    def _mock_methods(self):
        self.notes.get_or_create_note_by_title = MagicMock(
            return_value=Note(self.test_note)
        )
        self.notes.put = MagicMock(return_value=self.MockResponse("", 201))
        self.notes._do_unlock = MagicMock(return_value=self.MockResponse("", 200))

    def test_write_notes_lock(self):
        # Note locked by self user without force_unlock
        self._mock_methods()
        self.notes._do_lock = MagicMock(
            return_value=self.MockResponse(self.locked_note, 200)
        )
        self.notes.get_notes = MagicMock()
        with pytest.raises(LockedException):
            self.notes.write_note(text="content", force_unlock=False)
        assert self.notes.get_or_create_note_by_title.call_count == 1
        assert self.notes._do_lock.call_count == 1
        assert self.notes.put.call_count == 0
        assert self.notes._do_unlock.call_count == 0

        # Note locked by self user with force_unlock
        self._mock_methods()
        self.notes._do_lock = MagicMock(
            return_value=self.MockResponse(self.locked_note, 200)
        )
        self.notes.write_note(text="content", force_unlock=True)
        assert self.notes.get_or_create_note_by_title.call_count == 1
        assert self.notes._do_lock.call_count == 1
        assert self.notes.put.call_count == 1
        assert self.notes._do_unlock.call_count == 1

        # Note unlocked without force_unlock
        self._mock_methods()
        self.notes._do_lock = MagicMock(
            return_value=self.MockResponse(self.locked_note, 201)
        )
        self.notes.write_note(text="content", force_unlock=False)
        assert self.notes.get_or_create_note_by_title.call_count == 1
        assert self.notes._do_lock.call_count == 1
        assert self.notes.put.call_count == 1
        assert self.notes._do_unlock.call_count == 1

        # Note unlocked with force_unlock
        self._mock_methods()
        self.notes._do_lock = MagicMock(
            return_value=self.MockResponse(self.locked_note, 201)
        )
        self.notes.write_note("content", force_unlock=True)
        assert self.notes.get_or_create_note_by_title.call_count == 1
        assert self.notes._do_lock.call_count == 1
        assert self.notes.put.call_count == 1
        assert self.notes._do_unlock.call_count == 1

        # Note locked by other user without force_unlock
        self._mock_methods()
        exception = HTTPError("Raise for status 403")
        exception.response = self.MockResponse(self.locked_note, 403)
        self.notes._do_lock = MagicMock(side_effect=exception)

        with pytest.raises(LockedException) as e:
            self.notes.write_note(text="content", force_unlock=False)
        assert str(e.value) == "Cannot unlock. Locked by @timmi."
        assert self.notes.get_or_create_note_by_title.call_count == 1
        assert self.notes._do_lock.call_count == 1
        assert self.notes.put.call_count == 0
        assert self.notes._do_unlock.call_count == 0

        # Note locked by other user with force_unlock
        self._mock_methods()
        exception = HTTPError("Raise for status 403")
        exception.response = self.MockResponse(self.locked_note, 403)
        self.notes._do_lock = MagicMock(side_effect=exception)

        with pytest.raises(LockedException) as e:
            self.notes.write_note(text="content", force_unlock=True)
        assert str(e.value) == "Cannot unlock. Locked by @timmi."
        assert self.notes.get_or_create_note_by_title.call_count == 1
        assert self.notes._do_lock.call_count == 1
        assert self.notes.put.call_count == 0
        assert self.notes._do_unlock.call_count == 0

    def test_notes_api_init(self):
        # Test valid personal note
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["cli"] = {"private_note": True}
        try:
            n = NotesAPI(reptor=self.reptor)
            assert n.private_note
        except ValueError:
            self.fail("NotesAPI raised Error")

        # Test valid project note
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["cli"] = {"private_note": False}
        self.reptor._config._raw_config[
            "project_id"
        ] = "2b5de38d-2932-4112-b0f7-42c4889dd64d"
        try:
            n = NotesAPI(reptor=self.reptor)
            assert not n.private_note
        except ValueError:
            self.fail("NotesAPI raised Error")

        # Test missing project id and missing private_note
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["cli"] = {"private_note": False}
        self.reptor._config._raw_config["project_id"] = ""
        with pytest.raises(ValueError):
            NotesAPI(reptor=self.reptor)

        # Test missing server
        self.reptor._config._raw_config["server"] = ""
        self.reptor._config._raw_config["cli"] = {"private_note": True}
        with pytest.raises(ValueError):
            NotesAPI(reptor=self.reptor)



import sys
from unittest.mock import MagicMock, Mock, mock_open, patch

import pytest
from requests.exceptions import HTTPError
from contextlib import nullcontext
from reptor.api.manager import APIManager
from reptor.lib.reptor import Reptor

from ..Project import Project


class TestProject:
    @pytest.fixture(autouse=True)
    def setUp(self):
        reptor = Reptor()
        reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        reptor._config._raw_config[
            "project_id"
        ] = "8a6ebd7b-637f-4f38-bfdd-3e8e9a24f64e"
        reptor._api = APIManager(reptor=reptor)
        self.project = Project(reptor=reptor)

    @pytest.mark.parametrize(
        "filename,upload,expected_kwargs",
        [
            (
                "test.tar.gz",
                False,
                {
                    "content": b"binary-data",
                    "filename": "test.tar.gz",
                    "upload": False,
                    "stdout": False,
                },
            ),
            (
                "test.tar.gz",
                True,
                {
                    "content": b"binary-data",
                    "filename": "test.tar.gz",
                    "upload": True,
                    "stdout": False,
                },
            ),
            (
                "-",
                True,
                {
                    "content": b"binary-data",
                    "filename": "my-project.tar.gz",
                    "upload": True,
                    "stdout": True,
                },
            ),
        ],
    )
    def test_export_archive(self, filename, upload, expected_kwargs):
        self.project.deliver_file = MagicMock()
        archive_data = b"binary-data"
        self.project.reptor.api.projects.export = MagicMock(return_value=archive_data)
        self.project.reptor.api.projects.project = MagicMock()
        self.project.reptor.api.projects.project.name = "my-project"

        self.project._export_project(format="tar.gz", filename=filename, upload=upload)

        self.project.deliver_file.assert_called_once_with(**expected_kwargs)

    @pytest.mark.parametrize(
        "format,filename,expected_kwargs",
        [
            (
                "json",
                "my-project.json",
                {
                    "content": b'{\n  "a": "b"\n}',
                    "filename": "my-project.json",
                    "upload": False,
                    "stdout": False,
                },
            ),
            (
                "toml",
                "my-project.toml",
                {
                    "content": b'a = "b"\n',
                    "filename": "my-project.toml",
                    "upload": False,
                    "stdout": False,
                },
            ),
            (
                "yaml",
                "my-project.toml",
                {
                    "content": b"a: b\n",
                    "filename": "my-project.toml",
                    "upload": False,
                    "stdout": False,
                },
            ),
            ("unknown", "unknown.txt", {}),
        ],
    )
    def test_export_project(self, format, filename, expected_kwargs):
        class MockResponse:
            content = b"binary-data"

        self.project.reptor.api.projects.project = MagicMock()
        self.project.reptor.api.projects.project.to_dict = MagicMock(
            return_value={"a": "b"}
        )
        self.project.reptor.api.projects.project.name = "my-project"
        self.project.deliver_file = MagicMock()
        self.project.reptor.api.projects.post = MagicMock(return_value=MockResponse())

        with pytest.raises(ValueError) if format == "unknown" else nullcontext():
            assert (
                self.project._export_project(format=format, filename=filename) is None
            )
        if format != "unknown":
            self.project.deliver_file.assert_called_once_with(**expected_kwargs)
        else:
            self.project.deliver_file.assert_not_called()

    def test_render_error(self):
        class MockResponse:
            content = {
                "pdf": None,
                "messages": [
                    {
                        "level": "error",
                        "message": "Template compilation error: Element is missing end tag.",
                        "details": "details",
                        "location": {
                            "type": "design",
                            "id": "5755c339-9ca7-4f83-a251-5490cfcf67e7",
                            "name": "Demo",
                            "path": None,
                        },
                    },
                    {"level": "warning", "message": "Template Warning"},
                ],
            }

            def json(self):
                return self.content

        self.project.reptor.api.projects.check_report = MagicMock(return_value={})
        exception = HTTPError("Raise for rendering error")
        exception.response = MockResponse()
        self.project.reptor.api.projects.post = Mock(
            side_effect=exception,
            return_value=MockResponse(),
        )
        self.project.reptor.api.projects.project = MagicMock()
        self.project.reptor.api.projects.project.name = "error-project"
        self.project.reptor.api.projects.log.error = MagicMock()
        self.project.reptor.api.projects.log.warning = MagicMock()

        # Assert rendering errors are logged
        with pytest.raises(HTTPError):
            self.project._render_project()
        self.project.reptor.api.projects.log.error.assert_called_once()
        self.project.reptor.api.projects.log.warning.assert_called_once()

        # Assert reraise if no json message
        exception.response.json = MagicMock(side_effect=NotImplementedError)
        with pytest.raises(HTTPError):
            self.project._render_project()

    def test_deliver_file(self):
        content = b"binary-data"
        filename = "test.tar.gz"
        default_filename = "my-project.tar.gz"
        sys.stdout.buffer.write = MagicMock()
        self.project.reptor.api.notes.upload_file = MagicMock()

        # Stdout, no upload
        self.project.deliver_file(
            content=content,
            filename=default_filename,
            upload=False,
            stdout=True,
        )
        sys.stdout.buffer.write.assert_called_once_with(content)
        self.project.reptor.api.notes.upload_file.assert_not_called()

        # Stdout, upload
        sys.stdout.buffer.write = MagicMock()
        self.project.deliver_file(
            content=content, filename=default_filename, upload=True, stdout=True
        )
        sys.stdout.buffer.write.assert_called_once_with(content)
        self.project.reptor.api.notes.upload_file.assert_not_called()
        # self.project.reptor.api.notes.upload_file.assert_called_once_with(
        #    content=content, filename=default_filename, notetitle="Uploads"
        # )

        # File, no upload
        m = mock_open()
        with patch("builtins.open", m, create=True):
            sys.stdout.buffer.write = MagicMock()
            self.project.reptor.api.notes.upload_file = MagicMock()
            self.project.deliver_file(
                content=content,
                filename=filename,
                upload=False,
                stdout=False,
            )
            sys.stdout.buffer.write.assert_not_called()
            self.project.reptor.api.notes.upload_file.assert_not_called()
            m.assert_called_once_with(filename, "wb")
            handle = m()
            handle.write.assert_called_once_with(content)

        # Write to file
        m = mock_open()
        with patch("builtins.open", m, create=True):
            sys.stdout.buffer.write = MagicMock()
            self.project.reptor.api.notes.upload_file = MagicMock()
            self.project.deliver_file(
                content=content,
                filename=filename,
                upload=False,
                stdout=False,
            )
            sys.stdout.buffer.write.assert_not_called()
            self.project.reptor.api.notes.upload_file.assert_not_called()
            m.assert_called_once_with(filename, "wb")
            handle = m()
            handle.write.assert_called_once_with(content)

        # File with default filename, upload
        m = mock_open()
        with patch("builtins.open", m, create=True):
            sys.stdout.buffer.write = MagicMock()
            self.project.reptor.api.notes.upload_file = MagicMock()
            self.project.deliver_file(
                content=content,
                filename=default_filename,
                upload=True,
                stdout=False,
            )
            sys.stdout.buffer.write.assert_not_called()
            self.project.reptor.api.notes.upload_file.assert_called_once_with(
                content=content, filename=default_filename, notetitle="Uploads"
            )
            m.assert_not_called()



import json

import pytest
import tomli_w

from reptor.lib.reptor import Reptor

from ..Finding import Finding


class TestFinding:
    finding_json = """{
	"status": "in-progress",
	"data": {
		"cvss": "CVSS:3.1/AV:N/AC:L/PR:H/UI:N/S:U/C:L/I:L/A:N",
		"title": "Reflected XSS",
		"summary": "We detected a reflected XSS vulnerability.\n\n**XSS targets**\n\n* https://example.com/alert(1)\n* https://example.com/q=alert(1)\n",
		"references": [
			"https://owasp.org/www-community/attacks/xss/"
		],
		"description": "",
		"recommendation": "HTML encode user-supplied inputs.",
		"affected_components": [
			"https://example.com/alert(1)",
			"https://example.com/q=alert(1)"
		]
	}
}
"""
    finding_toml = tomli_w.dumps(json.loads(finding_json, strict=False))

    @pytest.fixture(autouse=True)
    def setup(self):
        self.reptor = Reptor()
        self.finding = Finding(reptor=self.reptor)

    def test_read_invalid_finding(self):
        with pytest.raises(ValueError):
            list(self.finding._read_findings(content="invalid"))

    def test_read_finding_with_invalid_field(self):
        invalid_finding = json.loads(self.finding_json, strict=False)
        invalid_finding["data"]["title"] = 1
        with pytest.raises(ValueError):
            list(self.finding._read_findings(content=json.dumps(invalid_finding)))

    def test_read_toml_finding(self):
        finding_content = list(self.finding._read_findings(content=self.finding_toml))[
            0
        ]
        assert isinstance(finding_content, dict)
        assert finding_content["data"]["title"] == "Reflected XSS"
        assert finding_content["data"]["summary"].startswith(
            "We detected a reflected XSS vulnerability."
        )

    def test_read_json_finding(self):
        finding_content = list(self.finding._read_findings(content=self.finding_json))[
            0
        ]
        assert isinstance(finding_content, dict)
        assert finding_content["data"]["title"] == "Reflected XSS"
        assert finding_content["data"]["summary"].startswith(
            "We detected a reflected XSS vulnerability."
        )

    def test_read_multiple_json_findings(self):
        findings = f"[{self.finding_json}, {self.finding_json}, {self.finding_json}]"
        finding_content = list(self.finding._read_findings(content=findings))
        assert len(finding_content) == 3

        finding_content = finding_content[0]
        assert isinstance(finding_content, dict)
        assert finding_content["data"]["title"] == "Reflected XSS"
        assert finding_content["data"]["summary"].startswith(
            "We detected a reflected XSS vulnerability."
        )



from ..Note import Note, NoteTemplate


class TestNoteModel:
    def test_from_note_template(self):
        note_template = NoteTemplate(
            data={
                "title": "Note Title",
                "text": "Note Text",
                "checked": True,
                "icon_emoji": "📤",
                "status_emoji": "📤",
                "order": 1,
                "parent": "abc-def",
            }
        )
        note = Note.from_note_template(note_template)
        assert note.title == "Note Title"
        assert note.text == "Note Text"
        assert note.checked is True
        assert note.icon_emoji == "📤"
        assert note.status_emoji == "📤"
        assert note.order == 1
        assert note.parent == "abc-def"

    def test_from_kwargs(self):
        note_template = NoteTemplate.from_kwargs(
            title="Note Title",
            text="Note Text",
            checked=True,
            icon_emoji="📤",
            status_emoji="📤",
            order=1,
            parent="abc-def",
        )
        assert note_template.title == "Note Title"
        assert note_template.text == "Note Text"
        assert note_template.checked is True
        assert note_template.icon_emoji == "📤"
        assert note_template.status_emoji == "📤"
        assert note_template.order == 1
        assert note_template.parent == "abc-def"



import csv
import io
import json
import typing

import cvss
import tomli_w
import yaml

from reptor.lib.plugins.Base import Base


class ProjectFindings(Base):
    """
    Export your project findings as a summary or checklist.
    """

    meta = {
        "name": "ProjectFindings",
        "summary": "Export your project findings as a summary or checklist",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.format: str = kwargs.get("export", "")
        self.output: typing.Optional[str] = kwargs.get("output")
        self.upload: bool = kwargs.get("upload", False)
        self.fieldnames: str = kwargs.get("fieldnames", "").split(",")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        parser.add_argument(
            "--format",
            help="Output format",
            choices=["csv", "json", "toml", "yaml"],
            type=str.lower,
            action="store",
            dest="export",
            default="csv",
        )
        parser.add_argument(
            "--fieldnames",
            help="Fieldnames to be included, comma-separated",
            action="store",
            default="retest_status,title,affected_components,cvss",
        )
        parser.add_argument(
            "-o",
            "--output",
            metavar="FILENAME",
            help="Filename to store output, empty for stdout",
            action="store",
            default=None,
        )
        parser.add_argument(
            "--upload",
            action="store_true",
            help="Used with --export or --render; uploads file to note",
            dest="upload",
        )

    def _findings_summary(self, format="csv", filename=None, upload=False):
        stdout = False
        default_filename = self.reptor.api.projects.project.name or "project"
        if filename in ["-", None] and not upload:
            stdout = True
            filename = None

        findings = list()
        for finding in self.reptor.api.projects.project.findings:
            finding_summary = dict()
            for field in self.fieldnames:
                try:
                    finding_summary[field] = getattr(finding.data, field).value
                except AttributeError:
                    finding_summary[field] = ""

                # Retest status if empty
                if field == "retest_status" and not finding_summary[field]:
                    finding_summary[field] = "Open"

                # Calculate CVSS
                if field == "cvss":
                    try:
                        finding_summary[field] = cvss.CVSS3(
                            finding_summary[field]
                        ).severities()[-1]
                    except (cvss.exceptions.CVSS3MalformedError, IndexError):
                        try:
                            finding_summary[field] = cvss.CVSS2(
                                finding_summary[field]
                            ).severities()[-1]
                        except (cvss.exceptions.CVSS2MalformedError, IndexError):
                            pass
                    # If calcaluation fails, the CVSS vector is kept as value

                # Join lists
                if isinstance(finding_summary[field], list):
                    finding_summary[field] = ", ".join(
                        f.value for f in finding_summary[field]
                    )
            findings.append(finding_summary)

        output = ""
        if format == "json":
            output = json.dumps(findings, indent=2)
        elif format == "toml":
            output = tomli_w.dumps(findings)  # type: ignore
        elif format == "yaml":
            output = yaml.dump(findings)
        elif format == "csv":
            with io.StringIO() as csvfile:
                writer = csv.DictWriter(csvfile, fieldnames=self.fieldnames)
                writer.writeheader()
                for row in findings:
                    writer.writerow(row)
                output = csvfile.getvalue()
        else:
            raise ValueError(f"Unknown format: {format}")
        self.deliver_file(
            content=output.encode(),
            filename=filename or f"{default_filename}.{format}",
            upload=upload,
            stdout=stdout,
        )

    def run(self):
        self._findings_summary(self.format, filename=self.output, upload=self.upload)


loader = ProjectFindings



import io
import json
import subprocess
import tarfile

import pytest
import yaml


@pytest.mark.integration
class TestIntegrationTemplate(object):
    def test_template_export_archive(self):
        p = subprocess.Popen(
            ["reptor", "template", "--search", "XE", "--export", "json"],
            stdout=subprocess.PIPE,
        )
        templates, _ = p.communicate()
        templates = json.loads(templates.decode())
        assert p.returncode == 0
        ids = [template["id"] for template in templates]

        p = subprocess.Popen(
            [
                "reptor",
                "template",
                "--search",
                "XE",
                "--export",
                "tar.gz",
                "--output",
                "-",
            ],
            stdout=subprocess.PIPE,
        )
        archive, _ = p.communicate()
        archive_io = io.BytesIO(archive)
        with tarfile.open(fileobj=archive_io, mode="r:gz") as tar:
            assert all(f"{id}.json" in list(tar.getnames()) for id in ids)
            assert len(tar.getmembers()) == len(ids)

    def test_template_export(self):
        p = subprocess.Popen(
            ["reptor", "template", "--export", "json"],
            stdout=subprocess.PIPE,
        )
        templates, _ = p.communicate()
        templates = json.loads(templates.decode())
        assert p.returncode == 0
        assert len(templates) > 0

        p = subprocess.Popen(
            ["reptor", "template", "--search", "SQL", "--export", "json"],
            stdout=subprocess.PIPE,
        )
        sql_templates, _ = p.communicate()
        sql_templates = json.loads(sql_templates.decode())
        assert p.returncode == 0
        assert len(templates) > len(sql_templates) > 0
        sql_summary = sql_templates[0]["translations"][0]["data"]["summary"]

        p = subprocess.Popen(
            ["reptor", "template", "--search", "SQL", "--export", "plain"],
            stdout=subprocess.PIPE,
        )
        sql_templates, _ = p.communicate()
        sql_templates = sql_templates.decode()
        assert p.returncode == 0
        assert sql_summary in sql_templates

        p = subprocess.Popen(
            ["reptor", "template", "--search", "SQL", "--export", "yaml"],
            stdout=subprocess.PIPE,
        )
        sql_templates, _ = p.communicate()
        sql_templates = yaml.safe_load(sql_templates.decode())
        assert p.returncode == 0
        assert len(templates) > len(sql_templates) > 0



from reptor.lib.plugins.ToolBase import ToolBase
from reptor.models.Note import NoteTemplate


class Nmap(ToolBase):
    """
    Author: Syslifters
    Version: 1.0
    Website: https://github.com/Syslifters/reptor
    License: MIT
    Tags: network,scanning,infrastructure

    Short Help:
    Formats nmap output

    Description:
    target=127.0.0.1
    sudo nmap -Pn -n -sV -oG - -p 1-65535 $target

    sudo -v  # Elevate privileges for non-interactive
    sudo -n nmap -Pn -n -sV -oG - -p 80,440 $target | tee nmap_result.txt | reptor nmap

    # Format and upload
    cat nmap_result.txt | reptor nmap -c upload
    """

    meta = {
        "name": "Nmap",
        "summary": "format nmap output",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.note_icon = "👁️‍🗨️"
        self.notetitle = kwargs.get("notetitle") or "Nmap"
        if self.input_format == "raw":
            self.input_format = "xml"

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        input_format_group = cls.get_input_format_group(parser)
        input_format_group.add_argument(
            "-oX",
            help="nmap XML output format, same as --xml (recommended)",
            action="store_const",
            dest="format",
            const="xml",
        )
        input_format_group.add_argument(
            "-oG",
            "--grepable",
            help="nmap Grepable output format",
            action="store_const",
            dest="format",
            const="grepable",
        )

    def parse_grepable(self):
        self.parsed_input = list()
        for line in self.raw_input.splitlines():
            if line.startswith("#") or "Ports:" not in line:
                continue
            ip, ports = line.split("Ports:")
            ip = ip.split(" ")[1]

            ports = ports.split(",")
            for port in ports:
                port, status, protocol, _, service, _, version, _ = port.strip().split(
                    "/"
                )
                if status == "open":
                    self.parsed_input.append(
                        {
                            "ip": ip,
                            "port": port,
                            "hostname": "",  # No hostname in greppable output
                            "protocol": protocol,
                            "service": service.replace("|", "/"),
                            "version": version.replace("|", "/"),
                        }
                    )

    def parse_xml(self):
        super().parse_xml()
        nmap_data = self.parsed_input
        self.parsed_input = list()
        hosts = nmap_data.get("nmaprun", {}).get("host", [])
        if not isinstance(hosts, list):
            hosts = [hosts]
        for host in hosts:
            ip = host.get("address", {}).get("@addr")
            ports = host.get("ports", {}).get("port", [])
            if not isinstance(ports, list):
                ports = [ports]
            for port in ports:
                if port.get("state", {}).get("@state") == "open":
                    self.parsed_input.append(
                        {
                            "ip": ip,
                            "hostname": (host.get("hostnames") or {})
                            .get("hostname", {})
                            .get("@name"),
                            "port": port.get("@portid"),
                            "protocol": port.get("@protocol"),
                            "service": port.get("service", {}).get("@name"),
                            "version": port.get("service", {}).get("@product"),
                        }
                    )

    def parse(self):
        super().parse()
        if self.input_format == "grepable":
            self.parse_grepable()

    def preprocess_for_template(self) -> dict:
        show_hostname = any([s.get("hostname") for s in self.parsed_input])
        return {"data": self.parsed_input, "show_hostname": show_hostname}

    def create_notes(self):
        data = dict()
        # Group by IP
        for ports in self.parsed_input:
            if ports["ip"] not in data:
                data[ports["ip"]] = list()
            data[ports["ip"]].append(ports)

        # Create note structure
        ## Main note with table of all IPs
        main_note = NoteTemplate()
        main_note.title = self.notetitle
        main_note.icon_emoji = self.note_icon
        main_note.template = self.template
        main_note.template_data = self.preprocess_for_template()
        ## Subnotes per IP
        for ip, ports in data.items():
            ip_note = NoteTemplate()
            ip_note.title = ip
            ip_note.checked = False
            ip_note.template = self.template
            ip_note.template_data = {"data": ports}
            ip_note.template_data["show_hostname"] = any(
                [p.get("hostname") for p in ports]
            )
            main_note.children.append(ip_note)
        return main_note


loader = Nmap



import contextlib
import typing
from datetime import datetime
from os.path import basename
from posixpath import join as urljoin

from requests import HTTPError

from reptor.api.APIClient import APIClient
from reptor.lib.exceptions import LockedException
from reptor.models.Note import Note
from reptor.utils.file_operations import guess_filetype

from reptor.models.Note import NoteTemplate


class NotesAPI(APIClient):
    """Interacts with Notes Endpoints

    Args:
        APIClient (_type_): _description_
    """

    def __init__(self, **kwargs) -> None:
        super().__init__(require_project_id=False, **kwargs)

        if self.private_note:
            self.base_endpoint = urljoin(
                self.reptor.get_config().get_server(),
                f"api/v1/pentestusers/self/notes/",
            )
        elif self.project_id:
            self.base_endpoint = urljoin(
                self.reptor.get_config().get_server(),
                f"api/v1/pentestprojects/{self.project_id}/notes/",
            )

    @property
    def private_note(self):
        return self.reptor.get_config().get_cli_overwrite().get("private_note")

    def get_notes(self) -> typing.List[Note]:
        """Gets list of notes"""
        self.debug("Getting Notes List")
        response = self.get(self.base_endpoint)
        notes = list()
        for note_data in response.json():
            notes.append(Note(note_data))
        return notes

    def get_note(
        self,
        noteid: typing.Optional[str] = None,
        notetitle: typing.Optional[str] = None,
    ) -> typing.Optional[Note]:
        for note in self.get_notes():
            if noteid:
                if note.id == noteid:
                    return note
            elif notetitle:
                if note.title == notetitle:
                    return note
            else:
                raise ValueError("Either noteid or notetitle must be provided")

    def create_note(
        self,
        title="CLI Note",
        parent_id: typing.Optional[str] = None,
        order=None,
        checked=None,
        icon=None,
    ) -> Note:
        self.debug("Creating Note")
        if title is None:
            raise ValueError("Note title must not be null.")
        note = self.post(
            self.base_endpoint,
            json={
                "order": order,
                "parent": parent_id or None,
                "checked": checked,
                "title": title,
            },
        ).json()
        self.debug(f"We created note with {note}")
        if icon:
            self.set_icon(note.get("id"), icon)
        return Note(note)

    def delete_note(self, notes_id: str):
        url = urljoin(self.base_endpoint, f"{notes_id}/")
        self.delete(url)

    def set_icon(self, notes_id: str, icon: str):
        url = urljoin(self.base_endpoint, f"{notes_id}/")
        self.put(url, json={"icon_emoji": icon})

    def _upload_note(
        self,
        note: Note,
        force_unlock: bool = False,
    ):
        with self._lock_note(note.id, force_unlock=force_unlock):
            self.debug(f"Note has length: {len(note.text)}")
            url = urljoin(self.base_endpoint, note.id, "")
            r = self.put(url, json=note.to_dict())

            try:
                r.raise_for_status()
                self.info(f'Note written to "{note.title}".')
            except HTTPError as e:
                raise HTTPError(
                    f'{str(e)} Are you uploading binary content to note? (Try "file" subcommand)'
                ) from e

    def write_note_templates(
        self,
        note_templates: typing.Union[NoteTemplate, typing.List[NoteTemplate]],
        timestamp: bool = True,
        **kwargs,
    ):
        if not isinstance(note_templates, list):
            note_templates = [note_templates]
        for note_template in note_templates:
            if note_template.parent_notetitle and not note_template.parent:
                note_template.parent = self.get_or_create_note_by_title(
                    note_template.parent_notetitle
                ).id

            note = self.get_note_by_title(
                note_template.title,
                parent_noteid=note_template.parent,
            )
            if note is None:
                new_note = True
                note = self.get_or_create_note_by_title(
                    title=note_template.title,
                    parent_noteid=note_template.parent,
                    parent_notetitle=note_template.parent_notetitle,
                )
            else:
                new_note = False
            self.debug(f"Got note from server {note.title} ({note.id})")

            # Prepare note (append content to existing note, etc)
            note_text = note.text + "\n\n" if note.text else ""
            if (
                timestamp and note_template.text
            ):  # Only add timestamp if there is content
                note_text += f"[{datetime.now().strftime('%Y-%m-%d %H:%M:%S')}]"
                if "\n" in note_template.text:
                    if not note_template.text.startswith("\n"):
                        note_text += "\n"
                else:
                    note_text += ": "
            note_text += note_template.text

            if new_note:
                upload_note = Note.from_note_template(note_template)
                upload_note.parent = note.parent
                upload_note.id = note.id
                upload_note.text = note_text
            else:
                upload_note = note
                upload_note.text = note_text

            # Upload note and children recursively
            self._upload_note(upload_note, **kwargs)
            for child in note_template.children:
                child.parent = note.id
                self.write_note_templates(child, timestamp=timestamp, **kwargs)

    def write_note(
        self,
        timestamp: bool = False,
        force_unlock: bool = False,
        **kwargs,
    ):
        note_template = NoteTemplate.from_kwargs(**kwargs)
        self.write_note_templates(
            note_template, timestamp=timestamp, force_unlock=force_unlock
        )

    def get_note_by_title(
        self,
        title,
        parent_noteid=None,  # Preferred over parent_notetitle
        parent_notetitle=None,
        ignore_parent=False,
    ) -> typing.Optional[Note]:
        if not parent_noteid and parent_notetitle:
            try:
                parent_noteid = self.get_note_by_title(parent_notetitle, ignore_parent=True).id  # type: ignore
            except AttributeError:
                raise ValueError(f'Parent note "{parent_notetitle}" does not exist.')
        notes_list = self.get_notes()

        for note in notes_list:
            if note.title == title and (note.parent == parent_noteid or ignore_parent):
                break
        else:
            return None
        return note

    def get_or_create_note_by_title(
        self,
        title,
        parent_noteid=None,  # Preferred over parent_notetitle
        parent_notetitle=None,
        icon=None,
    ) -> Note:
        if not parent_noteid and parent_notetitle:
            parent_noteid = self.get_or_create_note_by_title(
                parent_notetitle, icon=icon
            ).id
        note = self.get_note_by_title(title, parent_noteid=parent_noteid)
        if not note:
            # Note does not exist. Create.
            if title == "Uploads":
                icon = "📤"
            note = self.create_note(title=title, parent_id=parent_noteid, icon=icon)
        return note

    def upload_file(
        self,
        file: typing.Optional[typing.IO] = None,
        content: typing.Optional[bytes] = None,
        notetitle: typing.Optional[str] = None,
        filename: typing.Optional[str] = None,
        caption: typing.Optional[str] = None,
        parent_notetitle: typing.Optional[str] = None,
        force_unlock: bool = False,
        **kwargs,
    ):
        assert file or content
        assert not (file and content)
        if notetitle is None:
            notetitle = "Uploads"

        if file:
            if file.name == "<stdin>":
                self.display("Reading from stdin...")
            elif not filename:
                filename = basename(file.name)
            # TODO this might be streamed to not load entire file to memory
            try:
                content = file.buffer.read()  # type: ignore
            except AttributeError:
                content = file.read()
            if not filename:
                filetype = guess_filetype(content) or "dat"
                filename = f"data.{filetype}"

            if not content:
                self.warning(f"{file.name} is empty. Will not upload.")
                return

        # Lock during upload to prevent unnecessary uploads and for endpoint setup
        note = self.get_or_create_note_by_title(
            notetitle, parent_notetitle=parent_notetitle
        )
        if self.private_note:
            url = urljoin(self.base_endpoint, "upload/")
        else:
            url = urljoin(self.base_endpoint.rsplit("/", 2)[0], "upload/")
        with self._lock_note(note.id, force_unlock=force_unlock):
            response_json = self.post(
                url, files={"file": (filename, content)}, json_content=False
            ).json()
            is_image = True if response_json.get("resource_type") == "image" else False
            if is_image:
                file_path = f"/images/name/{response_json['name']}"
                note_content = f"\n![{caption or filename}]({file_path})"
            else:
                file_path = f"/files/name/{response_json['name']}"
                note_content = f"\n[{caption or filename}]({file_path})"

            self.write_note(
                title=notetitle,
                text=note_content,
                parent_notetitle=parent_notetitle,
                force_unlock=True,
                **kwargs,
            )

    def _do_lock(self, note_id: str):
        url = urljoin(self.base_endpoint, note_id, "lock/")
        return self.post(url)

    def _do_unlock(self, note_id: str):
        url = urljoin(self.base_endpoint, note_id, "unlock/")
        return self.post(url)

    @contextlib.contextmanager
    def _lock_note(self, note_id: str, force_unlock: bool = False):
        try:
            r = self._do_lock(note_id)
        except HTTPError as e:
            if e.response.status_code == 403:
                locked_by = (
                    e.response.json()
                    .get("lock_info", {})
                    .get("user", {})
                    .get("username")
                )
                if locked_by:
                    raise LockedException(f"Cannot unlock. Locked by @{locked_by}.")
                else:
                    raise LockedException("Cannot unlock. Locked by other user.")
            else:
                raise e
        if not force_unlock and r.status_code == 200:
            raise LockedException(
                "This note is locked. (Unlock or force: --force-unlock)"
            )

        yield

        self._do_unlock(note_id)



import os
import subprocess
from pathlib import Path

import pytest

from reptor.plugins.core.Conf.tests.conftest import notes_api, projects_api


@pytest.mark.integration
class TestIntegrationSslyze(object):
    def test_sslyze_note(self, notes_api):
        input_path = Path(os.path.dirname(__file__)) / "data/sslyze.json"

        p = subprocess.Popen(
            ["reptor", "sslyze", "--upload"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        p.communicate(input=input_path.read_bytes())
        assert p.returncode == 0

        note = notes_api.get_note_by_title(
            "🚩 www.example.com:443 (127.0.0.1)", parent_notetitle="Sslyze"
        )
        note_lines = note.text.splitlines()
        lines = [
            ' * <span style="color: red">SSLv2</span>',
            ' * <span style="color: green">TLS 1.2</span>',
            ' * SHA1 in certificate chain: <span style="color: green">No</span>',
            ' * Heartbleed: <span style="color: red">Yes</span>',
            ' * No Secure Renegotiation: <span style="color: red">Yes</span>',
            ' * <span style="color: orange">DHE-RSA-AES256-SHA256</span> (TLS 1.2)',
        ]
        assert all([line in note_lines for line in lines])

    def test_sslyze_findings(self, projects_api):
        nmap_output_path = Path(os.path.dirname(__file__)) / "data/sslyze.json"

        p = subprocess.Popen(
            ["reptor", "sslyze", "--push-findings"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        p.communicate(input=nmap_output_path.read_bytes())
        assert p.returncode == 0

        projects_api.get_findings()
        assert "Weak TLS setup might impact encryption" in [
            f.data.title for f in projects_api.get_findings()
        ]



import os
import subprocess
from pathlib import Path

import pytest

from reptor.plugins.core.Conf.tests.conftest import notes_api


@pytest.mark.integration
class TestIntegrationNmap(object):
    def test_nmap_note(self, notes_api):
        nmap_output_path = (
            Path(os.path.dirname(__file__)) / "data/nmap_multi_target.xml"
        )

        p = subprocess.Popen(
            ["reptor", "nmap", "-oX", "--upload"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, err = p.communicate(input=nmap_output_path.read_bytes())
        assert p.returncode == 0

        nmap_note = notes_api.get_note_by_title("Nmap", parent_notetitle=None)
        note_lines = nmap_note.text.splitlines()
        assert (
            "| www.google.com | 142.250.180.228 | 443/tcp | https | gws |" in note_lines
        )
        assert (
            "| www.syslifters.com | 34.249.200.254 | 443/tcp | https | n/a |"
            in note_lines
        )

        ip_note = notes_api.get_note_by_title(
            "142.250.180.228", parent_notetitle="Nmap"
        )
        note_lines = ip_note.text.splitlines()
        assert (
            "| www.google.com | 142.250.180.228 | 443/tcp | https | gws |" in note_lines
        )
        assert (
            "| www.syslifters.com | 34.249.200.254 | 443/tcp | https | n/a |"
            not in note_lines
        )

        ip_note = notes_api.get_note_by_title("34.249.200.254", parent_notetitle="Nmap")
        note_lines = ip_note.text.splitlines()
        assert (
            "| www.google.com | 142.250.180.228 | 443/tcp | https | gws |"
            not in note_lines
        )
        assert (
            "| www.syslifters.com | 34.249.200.254 | 443/tcp | https | n/a |"
            in note_lines
        )
